@@ -1,9 +1,10 @@
 #!/usr/bin/perl
-#my $Id = '$Id: countub.pl 367 2008-12-18 14:44:21Z pro $';
+#my $Id = '$Id: countub.pl 432 2009-01-12 01:53:07Z pro $';
 
 =copyright
 counting users-bytes from dchub for mrtg or cacti (snmpd)
 =cut
+
 use strict;
 use lib '../lib';
 use Net::DirectConnect::clihub;
@@ -17,11 +18,10 @@ my $dc = Net::DirectConnect::clihub->new(
 );
 $dc->connect();
 #  $dc->cmd('GetNickList');
-#  $dc->recv();
 my ($share) = (0);
-$dc->wait_sleep(3);    #for 1 .. 3;
-$dc->cmd( 'GetINFO', $_ ) for grep !$dc->{'NickList'}->{$_}{'info'}, keys %{ $dc->{'NickList'} };
-$dc->wait_sleep(3);    #for 1 .. 3;
+$dc->wait_connect();    #for 1 .. 3;
+$dc->cmd('GetINFO');
+$dc->wait(3);
 $share += $dc->{'NickList'}{$_}{'sharesize'} for keys %{ $dc->{'NickList'} };
 $share /= $ARGV[2] if $ARGV[2];
 print( ( scalar keys %{ $dc->{'NickList'} } or 0 ), "\n$share\n$ARGV[0]\nz\n" );
