@@ -1,18 +1,14 @@
-#Copyright (C) 2005-2006 Oleg Alexeenkov http://sourceforge.net/projects/dcppp proler@gmail.com icq#89088275
-#my $Id = '$Id: clihub.pm 373 2008-12-20 20:28:43Z pro $';
+# $Id: clihub.pm 465 2009-01-28 18:54:10Z pro $ $URL: svn://svn.setun.net/dcppp/trunk/lib/Net/DirectConnect/clihub.pm $
 package Net::DirectConnect::clihub;
-#use lib '../../..';
-#use lib '../..';
-#use lib '..';
-#  use Time::HiRes;
-eval { use Time::HiRes qw(time sleep); };
+use Time::HiRes qw(time sleep);
 use Data::Dumper;    #dev only
 $Data::Dumper::Sortkeys = 1;
 use Net::DirectConnect;
 use Net::DirectConnect::clicli;
+#use Net::DirectConnect::http;
 use strict;
 no warnings qw(uninitialized);
-our $VERSION = ( split( ' ', '$Revision: 373 $' ) )[1];
+our $VERSION = ( split( ' ', '$Revision: 465 $' ) )[1];
 #our @ISA = ('Net::DirectConnect');
 use base 'Net::DirectConnect';
 #todo! move to main module
@@ -26,7 +22,8 @@ sub init {
     'host' => 'localhost',
     #        'myport' => 6779 + int(rand(1000)),
     #	'Version'	=> '++ V:0.673,M:A,H:0/1/0,S:2',
-    'Pass' => '', 'Key' => 'zzz',
+    'Pass' => '',
+    'key'  => 'zzz',
     #    'auto_wait'        => 1,
     #        %$self,
     'supports_avail' => [ qw(
@@ -64,28 +61,30 @@ sub init {
     #}
     ||= {
     'chatline' => sub {
-      #$self->log('dcdev', 'chatline parse', Dumper(@_));
-      my ( $nick, $text ) = $_[0] =~ /^<([^>]+)> (.+)$/;
-      #v: chatline <[++T]> You are already in the hub.
+      #      my ( $nick, $text ) = $_[0] =~ /^[*<]([^>]+?)>? (.+)$/s;
+      my ( $nick, $text ) = $_[0] =~ /^(?:<|\* )(.+?)>? (.+)$/s;
+  #      my ( $nick, $text ) ;( $nick, $text ) = $_[0] =~ /^<([^>]+)> (.+)$/s or ( $nick, $text ) = $_[0] =~ /^\* (\S+) (.+)$/s;
+  #      $self->log('dcdev', 'chatline parse', Dumper(\@_,$nick, $text));
+  #v: chatline <[++T]> You are already in the hub.
       $self->log( 'warn', "[$nick] oper: already in the hub [$self->{'Nick'}]" ), $self->cmd('nick_generate'),
         $self->reconnect(),
         if ( ( !keys %{ $self->{'NickList'} } or $self->{'NickList'}->{$nick}{'oper'} )
         and $text eq 'You are already in the hub.' );
       if ( $self->{'NickList'}->{$nick}{'oper'} or $nick eq 'Hub-Security' ) {
         if (
-             $text =~ /^Minimum search interval is:(\d+)s/
+             $text =~ /^(?:Minimum search interval is|  ):(\d+)s/
           or $text =~ /Search ignored\.  Please leave at least (\d+) seconds between search attempts\./  #Hub-Security opendchub
           )
         {
           $self->log( 'warn', "[$nick] oper: set min interval = $1" );
-          $self->{'search_every'} = $1 || $self->{'search_every_min'};
+          $self->{'search_every'} = int $1 || $self->{'search_every_min'};
           $self->search_retry();
         }
         if ( $text =~ /  (\d+)    \./
           or $text eq '     !' )
         {
-          $self->log( 'warn', "[$nick] oper: increase min interval += $1" ),
-            $self->{'search_every'} += $1 || $self->{'search_every_min'};
+          $self->log( 'warn', "[$nick] oper: increase min interval +=", int $1 || $self->{'search_every_min'} ),
+            $self->{'search_every'} += int $1 || $self->{'search_every_min'};
           $self->search_retry();
         }
       }
@@ -93,15 +92,30 @@ sub init {
       $self->search_retry(),
         if $self->{'NickList'}->{$nick}{'oper'} and $text eq 'Sorry Hub is busy now, no search, try later..';
     },    #print("welcome:", @_) unless $self->{'no_print_welcome'}; },
-    'welcome' => sub { },    #print("welcome:", @_)
+    'welcome' => sub {
+      my ( $nick, $text ) = $_[0] =~ /^(?:<|\* )(.+?)>? (.+)$/s;
+      #$nick, $text
+      if ( ( !keys %{ $self->{'NickList'} } or !exists $self->{'NickList'}->{$nick} or $self->{'NickList'}->{$nick}{'oper'} )
+        and $text =~ /^Bad nickname: unallowed characters, use these (\S+)/ )
+      {
+        my $try = $self->{'Nick'};
+        $try =~ s/[^\Q$1\E]//g;
+        $self->log( 'warn', "CHNICK $self->{'Nick'} -> $try" );
+        $self->{'Nick'} = $try if length $try;
+      }
+    },    #print("welcome:", @_)
     'Lock' => sub {
-      #print "lockparse[$_[0]]\n";
+      $self->log( "lockparse", @_ );
       $self->{'sendbuf'} = 1;
       $self->cmd('Supports');
       #        $_[0] =~ /EXTENDEDPROTOCOL::\S+::(CTRL\[[^\]]+)\]/ or $_[0] =~ /(\S+)/;
       $_[0] =~ /^(.+?)(\s+Pk=.+)?\s*$/is;
-      #print "lock[$1]\n";
-      $self->cmd( 'Key', Net::DirectConnect::lock2key($1) );
+      print "lock[$1]\n";
+      #my $k = $self->lock2key($1);
+      #      $self->log( "key=", $k);
+      #      $self->cmd( 'Key', $k );
+      $self->cmd( 'Key', $self->lock2key($1) );
+      #Net::DirectConnect::
       #	$self->cmd('Key', Net::DirectConnect::lock2key($_[0]));
       #!!!!!ALL $self->cmd
       $self->{'sendbuf'} = 0;
@@ -110,7 +124,7 @@ sub init {
     },
     'Hello' => sub {
       return unless $_[0] eq $self->{'Nick'};
-      #$self->{'log'}->('info', "HELLO recieved, connected.");
+      #$self->log('info', "HELLO recieved, connected.");
       $self->{'sendbuf'} = 1;
       $self->cmd('Version');
       $self->{'sendbuf'} = 0 unless $self->{'auto_GetNickList'};
@@ -119,7 +133,8 @@ sub init {
       $self->{'status'} = 'connected';
       #        $self->{'no_print_welcome'} = 1;
       #      	$self->wait();
-      #$self->{'log'}->('info', "HELLO end rec st:[$self->{'status'}]");
+      #$self->log('info', "HELLO end rec st:[$self->{'status'}]");
+      $self->cmd('make_hub');
     },
     'Supports' => sub {
       $self->supports_parse( $_[0], $self );
@@ -129,7 +144,7 @@ sub init {
       $self->cmd('ValidateNick');
     },
     'To' => sub {
-      $self->{'log'}->( 'msg', "Private message to", @_ );
+      $self->log( 'msg', "Private message to", @_ );
     },
     'MyINFO' => sub {
       my ( $nick, $info ) = $_[0] =~ /\S+\s+(\S+)\s+(.*)/;
@@ -162,9 +177,10 @@ sub init {
       $self->{'NickList'}->{$_}{'oper'} = 1 for grep $_, split /\$\$/, $_[0];
     },
     'ForceMove' => sub {
-      $self->{'log'}->( 'info', "ForceMove to $_[0]" );
+      $self->log( 'warn', "ForceMove to $_[0]" );
       $self->disconnect();
-      $self->connect(@_) if $self->{'follow_forcemove'};
+      sleep(1);
+      $self->connect(@_) if $self->{'follow_forcemove'} and @_;
     },
     'Quit' => sub {
       $self->{'NickList'}->{ $_[0] }{'online'} = 0;
@@ -176,7 +192,7 @@ sub init {
       #         $self->{'NickList'}->{$nick}{'ip'} = $hp;
       #         $self->{'IpList'}->{$hp} = \%{ $self->{'NickList'}->{$nick} };
       $self->{'PortList'}->{$host} = $port;
-      #$self->{'log'}->('dev', "portlist: $host = $self->{'PortList'}->{$host} :=$port");
+      #$self->log('dev', "portlist: $host = $self->{'PortList'}->{$host} :=$port");
       return if $self->{'clients'}{ $host . ':' . $port }->{'socket'};
       $self->{'clients'}{ $host . ':' . $port } = Net::DirectConnect::clicli->new(
         %$self, $self->clear(),
@@ -191,9 +207,10 @@ sub init {
 #'clients' => {},
 #'debug'=>1,
 #    'auto_listen' => 0,
+        'auto_connect' => 1,
       );
       #$self->log( 'cldmp',Dumper $self->{'clients'}{ $host . ':' . $port });
-      $self->{'clients'}{ $host . ':' . $port }->cmd('connect');
+      #      $self->{'clients'}{ $host . ':' . $port }->cmd('connect_aft');
     },
     'RevConnectToMe' => sub {
       my ( $to, $from ) = split /\s+/, $_[0];
@@ -222,13 +239,16 @@ sub init {
         ( $s{'ip'}, $s{'port'} ) = split /:/, $s{'who'};
       }
       #my ($tth, string);
-      if ( $s{'cmd'}[4] =~ /^TTH:(.*)$/ ) {
+      #      if ( $s{'cmd'}[4] =~ /^TTH:(.*)$/ ) {
+      if ( $s{'cmd'}[4] =~ /^TTH:([0-9A-Z]{39})$/ ) {
+        #      if ( $s{'cmd'}[4] =~ /^TTH:\w{39}$/ ) {
         $s{'tth'} = $1;
-        $s{'string'} = $s{'tth'}, $s{'tth'} = undef unless length $s{'tth'} == 39 and $s{'tth'} =~ /^[0-9A-Z]+$/;
+        #       $s{'string'} = $s{'tth'}, $s{'tth'} = undef unless length $s{'tth'} == 39 and $s{'tth'} =~ /^[0-9A-Z]+$/;
       } else {
         $s{'string'} = $s{'cmd'}[4];
       }
       $s{'string'} =~ tr/$/ /;
+      #$self->log('dcdev', 'separse',"[$s{'cmd'}[4]]",Dumper \%s);
       return \%s;
     },    #todo
     'SR' => sub {
@@ -266,7 +286,18 @@ sub init {
     #
     #      'UserIP' => sub { print"todo[UserIP]$_[0]\n"}, #todo
     #      'ConnectToMe' => sub { print"todo[ConnectToMe]$_[0]\n"}, #todo
-    'UserCommand' => sub { },    # useless
+    'UserCommand' => sub { },                          # useless
+                                                       #
+                                                       #
+                                                       #
+                                                       #
+                                                       # ADC dev
+    'ISUP'        => sub { },
+    'ISID'        => sub { $self->{'sid'} = $_[0] },
+    'IINF'        => sub { $self->cmd('BINF') },
+    #todo
+    'IQUI' => sub { },
+    'ISTA' => sub { $self->log( 'dcerr', @_ ) },
     };
   #  %{
   $self->{'cmd'}
@@ -276,9 +307,9 @@ sub init {
       for (@_) {
         #  	  sleep($self->{'min_chat_delay'}) if $self->{'min_chat_delay'};
         #          if ($self->{'min_chat_delay'}) {
-        return unless $self->{'socket'};
+        #        return unless $self->{'socket'};
         if ( $self->{'min_chat_delay'} and ( time - $self->{'last_chat_time'} < $self->{'min_chat_delay'} ) ) {
-          $self->{'log'}->( 'dbg', 'sleep', $self->{'min_chat_delay'} - time + $self->{'last_chat_time'} );
+          $self->log( 'dbg', 'sleep', $self->{'min_chat_delay'} - time + $self->{'last_chat_time'} );
           $self->wait_sleep( $self->{'min_chat_delay'} - time + $self->{'last_chat_time'} );
         }
         $self->{'last_chat_time'} = time;
@@ -289,7 +320,7 @@ sub init {
           "<$self->{'Nick'}> $_|",
           "]:", $self->{'socket'}->send("<$self->{'Nick'}> $_|"), $!
         );
-        #$self->{'log'}->('dbg', 'sleep', $self->{'min_chat_delay'}),
+        #$self->log('dbg', 'sleep', $self->{'min_chat_delay'}),
       }
     },
     #$To: <othernick> From: <nick> $<<nick>> <message>|
@@ -301,6 +332,8 @@ sub init {
       #	$self->{'socket'}->send('To :', "$to From: $self->{'Nick'} \$<$self->{'Nick'}> $_|") for(@_);
     },
     'Key' => sub {
+      my $self = shift if ref $_[0];
+#      $self->log( 'dev', "sendkey", $_[0] );
       $self->sendcmd( 'Key', $_[0] );
     },
     'ValidateNick' => sub {
@@ -314,17 +347,30 @@ sub init {
 #      'MyINFO'	=> sub { $self->sendcmd('MyINFO', '$ALL', $self->{'Nick'}, $self->{'MyINFO'}); },
 #      'MyINFO'	=> sub { $self->sendcmd('MyINFO', '$ALL', $self->{'Nick'}, $self->{'description'} . '$ $' . $self->{'connection'} . chr($self->{'flag'}) . '$' . $self->{'email'} . '$' . $self->{'sharesize'} . '$'); },
     'GetNickList' => sub { $self->sendcmd('GetNickList'); },
-    'GetINFO'     => sub { $self->sendcmd( 'GetINFO', $_[0], $self->{'Nick'} ); },
+    'GetINFO'     => sub {
+      my $self = shift if ref $_[0];
+      #$self->sendcmd( 'GetINFO', $_[0], $self->{'Nick'} ), return if scalar @_ == 1;
+      @_ = grep { $self->{'NickList'}{$_}{'online'} and !$self->{'NickList'}{$_}{'info'} } keys %{ $self->{'NickList'} }
+        unless @_;
+      local $self->{'sendbuf'} = 1;
+      $self->sendcmd( 'GetINFO', $_, $self->{'Nick'} ) for @_;
+      #$dc->{'sendbuf'} = 0;
+      $self->sendcmd();
+    },
     'ConnectToMe' => sub {
+      my $self = shift if ref $_[0];
       #print "ctm [$self->{'M'}][$self->{'allow_passive_ConnectToMe'}]\n";
       return if $self->{'M'} eq 'P' and !$self->{'allow_passive_ConnectToMe'};
-      $self->{'log'}->( 'err', "please define myip" ), return unless $self->{'myip'};
+      $self->log( 'err', "please define myip" ), return unless $self->{'myip'};
       $self->sendcmd( 'ConnectToMe', $_[0], "$self->{'myip'}:$self->{'myport'}" );
     },
     'RevConnectToMe' => sub {
+      my $self = shift if ref $_[0];
+      $self->log( "send", ( 'RevConnectToMe', $self->{'Nick'}, $_[0] ), ref $_[0] );
       $self->sendcmd( 'RevConnectToMe', $self->{'Nick'}, $_[0] );
     },
     'MyPass' => sub {
+      my $self = shift if ref $_[0];
       my $pass = ( $_[0] or $self->{'Pass'} );
       $self->sendcmd( 'MyPass', $pass ) if $pass;
     },
@@ -336,11 +382,13 @@ sub init {
       $self->disconnect();
     },
     'Search' => sub {
+      my $self = shift if ref $_[0];
       #$self->log('dcdev', 'Search', @_);
       $self->sendcmd( 'Search', ( $self->{'M'} eq 'P' ? "Hub:$self->{'Nick'}" : "$self->{'myip'}:$self->{'myport_udp'}" ),
         join '?', @_ );
     },
     'search_buffer' => sub {
+      my $self = shift if ref $_[0];
       #      $self->log('dcdev', 'Search_buffer', @_);
       #return;
       push( @{ $self->{'search_todo'} }, [@_] ) if @_;
@@ -361,11 +409,13 @@ sub init {
       $self->{'search_last_time'} = time();
     },
     'search_tth' => sub {
+      my $self = shift if ref $_[0];
       #      $self->Search(  'F', 'T', '0', '9', 'TTH:'.$_[0]);
       $self->{'search_last_string'} = undef;
       $self->cmd( 'search_buffer', 'F', 'T', '0', '9', 'TTH:' . $_[0] );
     },
     'search_string' => sub {
+      my $self = shift if ref $_[0];
       my $string = $_[0];
       $self->{'search_last_string'} = $string;
       $string =~ tr/ /$/;
@@ -373,10 +423,12 @@ sub init {
       $self->cmd( 'search_buffer', 'F', 'T', '0', '1', $string );
     },
     'search' => sub {
+      my $self = shift if ref $_[0];
       return $self->cmd( 'search_tth', @_ ) if length $_[0] == 39 and $_[0] =~ /^[0-9A-Z]+$/;
       return $self->cmd( 'search_string', @_ ) if length $_[0];
     },
     'search_retry' => sub {
+      my $self = shift if ref $_[0];
       #      $self->cmd( 'search_buffer', @{$self->{'search_last'}})
       #unshift( @{ $self->{'search_todo'} }, [@{$self->{'search_last'}}] )
       unshift( @{ $self->{'search_todo'} }, $self->{'search_last'} )
@@ -384,19 +436,63 @@ sub init {
       $self->{'search_last'} = undef;
     },
     'make_hub' => sub {
-      $self->{'hub'} ||= $self->{'host'} . ( $self->{'port'} and $self->{'port'} != 411 ? $self->{'port'} : '' );
+      $self->{'hub'} ||= $self->{'host'} . ( ( $self->{'port'} and $self->{'port'} != 411 ) ? ':' . $self->{'port'} : '' );
     },
     'nick_generate' => sub {
       $self->{'nick_base'} ||= $self->{'Nick'};
       $self->{'Nick'} = $self->{'nick_base'} . int( rand( $self->{'nick_random'} || 100 ) );
     },
+    #
+    #
+    #
+    # ADC dev
+    'connect_aft' => sub { $self->cmd('HSUP') if $self->{'protocol'} eq 'adc' },
+    'HSUP' => sub {
+      $self->{'SUPADS'} ||= [qw(BAS0 BASE TIGR UCM0 BLO0)];
+      $self->{'SUPAD'} ||= { map { $_ => 1 } @{ $self->{'SUPADS'} } };
+      $self->sendcmd( 'HSUP', ( map { 'AD' . $_ } @{ $self->{'SUPADS'} } ), ( map { 'RM' . $_ } keys %{ $self->{'SUPRM'} } ), );
+      #ADBAS0 ADBASE ADTIGR ADUCM0 ADBLO0
+    },
+    'BINF' => sub {
+      $self->{'BINFS'} ||= [qw(ID PD NI SL SS SF HN HR HO VE US SU)];
+      #$self->{'ID'} ||= 'FXC3WTTDXHP7PLCCGZ6ZKBHRVAKBQ4KUINROXXI';
+      #$self->{'PD'} ||='P26YAWX3HUNSTEXXYRGOIAAM2ZPMLD44HCWQEDY';
+      $self->{'NI'} ||= $self->{'Nick'} || 'perlAdcDev';
+      # hash() returns a 192 bit hash
+      #$self->log('TIGERTE',  Digest::Tiger::hash('Tiger'));
+      #$self->log('TIGERTEST',  MIME::Base32::encode(Digest::Tiger::hash('Tiger')));
+      sub hash {
+        local ($_) = @_;
+        eval "use MIME::Base32 qw( RFC ); use Digest::Tiger;";
+        #$_.=("\x00"x(1024 - length $_));print ( 'hlen', length $_);
+        MIME::Base32::encode( Digest::Tiger::hash($_) );
+      }
+      #$self->log('TIGERTEST',  hash('Tiger'));
+      #$self->log('TIGERTESTid',  hash('FXC3WTTDXHP7PLCCGZ6ZKBHRVAKBQ4KUINROXXI'));
+      #$self->log('TIGERTESTpd', hash('P26YAWX3HUNSTEXXYRGOIAAM2ZPMLD44HCWQEDY'));
+      $self->{'PD'} ||= hash( 'perl' . $self->{'myip'} . $self->{'NI'} . time );
+      $self->{'ID'} ||= hash( $self->{'PD'} );
+      $self->{'SL'} ||= $self->{'S'} || '2';
+      $self->{'SS'} ||= $self->{'sharesize'} || 20025693588;
+      $self->{'SF'} ||= 30999;
+      $self->{'HN'} ||= $self->{'H'} || 1;
+      $self->{'HR'} ||= $self->{'R'} || 0;
+      $self->{'HO'} ||= $self->{'O'} || 0;
+      $self->{'VE'} ||= $self->{'V'} || '++\s0.706';
+      $self->{'US'} ||= 10000;
+      $self->{'SU'} ||= 'ADC0';
+      #$self->{''} ||= $self->{''} || '';
+      $self->sendcmd( 'BINF', $self->{'sid'}, map { $_ . $self->{$_} } grep { $self->{$_} } @{ $self->{'BINFS'} } );
+      #BINF UUXX IDFXC3WTTDXHP7PLCCGZ6ZKBHRVAKBQ4KUINROXXI PDP26YAWX3HUNSTEXXYRGOIAAM2ZPMLD44HCWQEDY NI SL2 SS20025693588
+      #SF30999 HN2 HR0 HO0 VE++\s0.706 US5242 SUADC0
+      }
     };
 #print "[$self->{'number'}]BEF";print "[$_ = $self->{$_}]"for sort keys %$self;print "\n";
 #print "[$self->{'number'}]CLR";print "[$_ = $clear{$_}]"for sort keys %clear;print "\n";
 #    $self->{'clients'}{''} = $self->{'incomingclass'}->new( %$self, %clear, 'socket' => $_, 'LocalPort'=>$self->{'myport'}, 'want' => \%{$self->{'want'}},
 #print "Listen on port $self->{'myport'} \n";
-  $self->log( 'dev', "making listeners: tcp" );
   if ( $self->{'M'} eq 'A' ) {
+  $self->log( 'dev', "making listeners: tcp" );
     $self->{'clients'}{'listener_tcp'} = $self->{'incomingclass'}->new(
       %$self, $self->clear(),
       'want'     => \%{ $self->{'want'} },
@@ -426,7 +522,10 @@ sub init {
       #'debug'=>1,
       #'nonblocking' => 0,
       'parse' => {
-        'SR' => $self->{'parse'}{'SR'}
+        'SR'   => $self->{'parse'}{'SR'},
+        'UPSR' => sub {
+          #    $self->log( 'dev', "UPSR", @_ );
+        },
 #2008/12/14-13:30:50 [3] rcv: welcome UPSR FQ2DNFEXG72IK6IXALNSMBAGJ5JAYOQXJGCUZ4A NIsss2911 HI81.9.63.68:4111 U40 TRZ34KN23JX2BQC2USOTJLGZNEWGDFB327RRU3VUQ PC4 PI0,64,92,94,100,128,132,135 RI64,65,66,67,68,68,69,70,71,72
 #UPSR CDARCZ6URO4RAZKK6NDFTVYUQNLMFHS6YAR3RKQ NIAspid HI81.9.63.68:411 U40 TRQ6SHQECTUXWJG5ZHG3L322N5B2IV7YN2FG4YXFI PC2 PI15,17,20,128 RI128,129,130,131
 #$SR [Predator]Wolf DC++\Btyan Adams - Please Forgive Me.mp314217310 18/20TTH:G7DXSTGPHTXSD2ZZFQEUBWI7PORILSKD4EENOII (81.9.63.68:4111)
@@ -440,8 +539,29 @@ sub init {
     $self->log( 'err', "cant listen udp (search repiles)" )
       unless $self->{'myport_udp'};
   }
+
+=z
+  $self->log( 'dev', "making listeners: http" );
+    $self->{'clients'}{'listener_http'} = Net::DirectConnect::http->new(
+      %$self, $self->clear(),
+#      'want'     => \%{ $self->{'want'} },
+#      'NickList' => \%{ $self->{'NickList'} },
+#      'IpList'   => \%{ $self->{'IpList'} },
+##      'PortList' => \%{ $self->{'PortList'} },
+      'handler'  => \%{ $self->{'handler'} },
+      #    $self->{'clients'}{''} = $self->{'incomingclass'}->new( %$self, $self->clear(),
+      #'LocalPort'=>$self->{'myport'},
+      #'debug'=>1,
+      'auto_listen' => 1,
+    );
+    $self->{'myport_http'}  = $self->{'clients'}{'listener_http'}{'myport'};
+    $self->log( 'err', "cant listen http" )
+      unless $self->{'myport_http'};
+=cut
+
+
   #
-  #$self->log('dev', "listeners created"),
+#  $self->log('dev', "listeners created"),
   #  $self->{'clients'}{'listener'}->listen();
   #print "[$self->{'number'}]AFT";print "[$_ = $self->{$_}]"for sort keys %$self;print "\n";
 }
